# This is version 2.6
#
# 2.3 does not show up for some reason
# I reorganized the package for Version 2.0
# 2.1 added some functions I forgot to add
# 2.2 - 2.4 had to be made because I fucked the required packages up
#       Apparently, packages os and time could not be installed?
# 2.4 did not work because internal scripts could not import each other
# 2.5 fixed that
# 2.6 - I forgot to put everything into packages for all previous 2.x versions. Fixed that
